/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#if !defined(AFX_DNSSOCKET_H__98A94FC0_D2BC_4F00_9DF4_CC706F156B06__INCLUDED_)
#define AFX_DNSSOCKET_H__98A94FC0_D2BC_4F00_9DF4_CC706F156B06__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "ErrorHandler.h"

#include "UDPSocketAsync.h"

#include <string>
#include <deque>

class CThreadPool;
class CGenericCriticalSection;

namespace KomodiaDNS
{

#define INITIAL_TIMEOUT 5000

class CDNSManager;
class CDNSTCPSocket;
class CDNSUDPSocket;
class CDNSAnswers;

class CDNSSocket : public CErrorHandler
{
	friend class CDNSTCPSocket;
	friend class CDNSUDPSocket;
public:
	//Get the thread manager
	CThreadPool* GetThreadManager()const;

	//Receive for block sockets
	CDNSAnswers* Receive();

	//Initialize data
	BOOL Initialize();

	//Set multithreaded support
	void SetMultithreaded(int iThreadNumber);
	void SetMultithreaded(CThreadPool* pThreadManager);

	//When we recieve an event
	CDNSAnswers* SocketReceive(char* pBuffer,
							   int iBufferLengh,
							   BOOL bTCP=FALSE);

	//Parse the answer
	CDNSAnswers* ParseAnswer(char* pBuffer,
							 char* pOriginalBuffer,
							 int iBufferLengh,
							 BOOL bFromThread=FALSE,
							 BOOL bTCP=FALSE);

	//Set socket timeout
	BOOL SetConnectionTimeout(int iMS);

	//Send the buffer
	BOOL Send(const char *pBuffer,
			  unsigned short bufLen,
			  LPVOID lpLocalIDData);

	//Ctor and Dtor
	CDNSSocket(CDNSManager* pFather,
			   const std::string strDNSServer,
			   BOOL bTCP,
			   BOOL bAsync,
			   BOOL bAsyncConnect);
	virtual ~CDNSSocket();
private:
	//Our parse struct
	typedef struct _ParseThreadData
	{
		char*		pData;
		char*		pOriginalData;
		int			iDataSize;
		CDNSSocket*	pSocket;
		BOOL		bTCP;
	} ParseThreadData;

	//Our pending requests ?
	typedef struct _WaitingRequests
	{
		char*				pData;
		unsigned short		usSize;
		LPVOID				lpData;
	} WaitingRequests;

	//Connection status
	typedef enum _ConnectionStatus
	{
		csNone,
		csPending,
		csConnected,
		csError,
		csDisconnected
	} ConnectionStatus;

	//Our deque
	typedef std::deque<WaitingRequests> PendingDeque;
private:
	//No copy Ctor
	CDNSSocket(const CDNSSocket& rSocket);

	//No copy operator
	CDNSSocket& operator=(const CDNSSocket& rSocket);

	//Indicte errors and timeout
	void SocketError(int iErrorCode);
	void SocketConnected(int iErrorCode);
	void SocketClosed();

	//Set the connection status
	void SetConnectionStatus(ConnectionStatus aStatus);

	//Our thread proc, for the thread manager
	static void ThreadProc(LPVOID lpParam);

	//Add data to the buffer
	void AddData(const char* pData,
		   		 unsigned short usSize,
				 LPVOID	lpData);

	//Get data from the pending requests
	WaitingRequests GetData()const;

	//Flush the data
	BOOL Flush();

	//Proc for deleting the vector data
	static void DeleteProc(WaitingRequests& rRequest);

	//Sockets settings
	BOOL m_bUseTCP;
	BOOL m_bAsync;
	BOOL m_bAsyncConnect;

	//timeout counter
	int m_iTimeout;

	//Our father
	CDNSManager* m_pFather;

	//Our sockets
	CDNSTCPSocket* m_pTCP;
	CDNSUDPSocket* m_pUDP;

	//Our DNS server
	std::string m_strDNSServer;

	//Our worker thread manager
	CThreadPool* m_pThreadManager;

	//Our critical section
	CGenericCriticalSection* m_pCSection;

	//Our CS for the deque
	CGenericCriticalSection* m_pCSectionDeque;

	//Do we own the thread manager ?
	BOOL m_bOwners;

	//Our connection status
	ConnectionStatus m_aConnectionStatus;

	//Our deque
	mutable PendingDeque m_aPendingData;
};

//of namespace
}

#endif // !defined(AFX_DNSSOCKET_H__98A94FC0_D2BC_4F00_9DF4_CC706F156B06__INCLUDED_)
