/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#if !defined(AFX_PINGSOCKET_H__469EE2E5_24E2_4DAA_A2BD_4A7DE284F4D4__INCLUDED_)
#define AFX_PINGSOCKET_H__469EE2E5_24E2_4DAA_A2BD_4A7DE284F4D4__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "ICMPSocketAsync.h"

KOMODIA_NAMESPACE_START

class CPingSocket : public CICMPSocketAsync
{
public:
	//Set the ID
	void SetID(unsigned short usID);

	//Set the sequence
	void SetSequence(unsigned short usSequence);

	//Use the same ID
	void SetUseSameID(BOOL bSameID);

	//Get the data size
	int GetDataSize()const;

	//Get the TTL
	unsigned char GetTTL()const;

	//Get the time elapsed
	DWORD GetTimeElapsed()const;

	//Was it a timeout
	BOOL IsTimeout()const;

	//Get the replying address
	IP GetReplyingAddress()const;

	//Do the ping
	virtual BOOL Ping(const std::string& rDestinationAddress,
					  DWORD dwTimeout=3000);

	virtual BOOL Ping(IP aDestinationAddress,
					  DWORD dwTimeout=3000);

	//Is the ping done ?
	BOOL IsPingDone()const;

	//Ctor and Dtor
	CPingSocket();
	virtual ~CPingSocket();
protected:
	//Our socket events
	NO_OnSocketAccept
	NO_OnSocketConnect
	NO_OnSocketClose
	NO_OnSocketOOB
	NO_OnSocketWrite

	virtual BOOL OnSocketTimeout();
	virtual BOOL OnSocketReceive(int iErrorCode);

	//Events for user to overide
	//Ping is done
	//bTimeout - If a timeout occured
	virtual void PingDone(BOOL bTimeout);

	//Ping is done, and this is the pings data
	//aReplyingAddress - Address that sent the ping
	//ucTTL - Received TTL
	//dwTimeElapsed - Time it took to the ping to arrive (ms)
	//iDataSize - Size of the ping's data
	//usID - Ping ID
	//bTTLExpired - Was it TTL expired packet
	virtual void PingDoneData(IP aReplyingAddress,
							  unsigned char ucTTL,
							  DWORD dwTimeElapsed,
							  int iDataSize,
							  unsigned short usID,
							  BOOL bTTLExpired);

	//Is an ID OK
	virtual BOOL IsIDOK(unsigned short usID,
						unsigned short usSequence)const;
private:
	//Last tick the icmp was sent
	DWORD m_dwLastTick;
	
	//Is the ping done ?
	BOOL m_bPingDone;

	//Our last ping ID
	unsigned short m_usLastPingID;

	//And sequence
	unsigned short m_usLastPingSequence;

	//Our replying address
	IP m_aReplyingAddress;

	//Time it tool
	DWORD m_dwTimeElapsed;

	//The TTL
	unsigned char m_ucTTL;

	//Was it a timeout
	BOOL m_bTimeout;

	//Packet data size
	int m_iDataSize;

	//Do we use the same ID
	BOOL m_bSameID;

	//Do we have a timeout
	DWORD m_dwTimeout;
};

KOMODIA_NAMESPACE_END

#endif // !defined(AFX_PINGSOCKET_H__469EE2E5_24E2_4DAA_A2BD_4A7DE284F4D4__INCLUDED_)
