/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#include "stdafx.h"
#include "RealTimeTraceRouteSocket.h"

#include "ErrorHandlerMacros.h"
#include "ThreadPool.h"
#include "GenericCriticalSection.h"
#include "OSManager.h"

#include <atlbase.h>

#ifdef _MEMORY_DEBUG 
	#define new	   DEBUG_NEW  
	#define malloc DEBUG_MALLOC  
    static char THIS_FILE[] = __FILE__;  
#endif

KOMODIA_NAMESPACE_START

// ----------------------------- CRealTimeDNSManager start ---------------------

#define CRealTimeDNSManager_Class "CRealTimeDNSManager"

CRealTimeTraceRouteSocket::CRealTimeDNSManager::CRealTimeDNSManager(CRealTimeTraceRouteSocket* pFather,
																	const std::string strDNSServer,
																	BOOL bTCP) : CDNSManager(strDNSServer,
																							 bTCP,
																							 TRUE,
																							 TRUE),
																				 m_pFather(pFather)
{
	try
	{
		//Set our name
		SetName(CRealTimeDNSManager_Class);
	}
	ERROR_HANDLER("CRealTimeDNSManager")
}

CRealTimeTraceRouteSocket::CRealTimeDNSManager::~CRealTimeDNSManager()
{
}

void CRealTimeTraceRouteSocket::CRealTimeDNSManager::OnDNSError(int iErrorCode,
																LPVOID lpLocalIDData)
{
	try
	{
		//Adjust the resolvers
		m_pFather->AdjustResolvers(-1);

		//We are done
		m_pFather->DNSDone();
	}
	ERROR_HANDLER("OnDNSError")
}

void CRealTimeTraceRouteSocket::CRealTimeDNSManager::OnDNSTimeout(const KomodiaDNS::CDNSQuery& rQuery,
																  LPVOID lpLocalIDData)
{
	try
	{
		//Adjust the resolvers
		m_pFather->AdjustResolvers(-1);

		//We are done
		m_pFather->DNSDone();
	}
	ERROR_HANDLER("OnDNSTimeout")
}

void CRealTimeTraceRouteSocket::CRealTimeDNSManager::OnDNSReceive(KomodiaDNS::CDNSAnswers* pAnswers,
																  LPVOID lpLocalIDData)
{
	try
	{
		//Protect the answers
		std::auto_ptr<KomodiaDNS::CDNSAnswers> pProtection(pAnswers);

		//Are we done
		if (m_pFather->m_bTraceDone)
			return;

		//Is it resolved ?
		if (lpLocalIDData)
		{
			//Get it
			TraceRouteData* pData;
			pData=(TraceRouteData*)lpLocalIDData;

			//Remove ourselves
			pData->pDNS=NULL;

			//Check the data
			if (pAnswers->IsResolved())
				//Do we have an answer
				if (!pAnswers->GetAnswer().empty())
				{
					//Save it
					pData->sReverseDNS=pAnswers->GetAnswer();

					//We are sesolved
					pData->bResolved=TRUE;

					//Do we have an address
					if (pData->aReplyingAddress)
					{
						//Call for modify
						m_pFather->TraceDataForModify(pData,
													  TRUE);

						//Call father
						m_pFather->TraceData(pData,
											 TRUE);
					}
				}
				else
					ReportError("CRealTimeDNSManager","Received empty answer!");
		}

		//Adjust the resolvers
		m_pFather->AdjustResolvers(-1);

		//We are done
		m_pFather->DNSDone();
	}
	ERROR_HANDLER("OnDNSReceive")
}

// ----------------------------- CRealTimeDNSManager end ---------------------

#define CRealTimeTraceRouteSocket_Class "CRealTimeTraceRouteSocket"

unsigned short CRealTimeTraceRouteSocket::m_sPortStart=0;

CRealTimeTraceRouteSocket::CRealTimeTraceRouteSocket(std::string sDNSServer,
													 BOOL bDNSTcp,
													 BOOL bMultithreaded) : CPingSocket(),
																			m_bTraceDone(TRUE),
																			m_pManager(NULL),
																			m_pCSection(NULL),
																			m_bInitialized(FALSE),
																			m_bResolveIPs(TRUE),
																			m_pCSectionResolve(NULL),
																			m_pCSectionGeneral(NULL),
																			m_bFinishing(FALSE),
																			m_bPartialDone(FALSE)
{
	try
	{
		//Set our name
		SetName(CRealTimeTraceRouteSocket_Class);

		//Same ping ID
		SetUseSameID(TRUE);

		//Create the manager
		m_pManager=new CRealTimeDNSManager(this,
										   sDNSServer,
										   bDNSTcp);

		//Create the CS
		m_pCSection=COSManager::CreateCriticalSection();
		m_pCSectionResolve=COSManager::CreateCriticalSection();
		m_pCSectionGeneral=COSManager::CreateCriticalSection();

		//Initialize it
		if (!(m_bInitialized=m_pManager->Initialize()))
			//Report it
			ReportError("CRealTimeTraceRouteSocket","Failed to initialize DNS!");
		else 
		{
			//Set the timeout of the DNS
			m_pManager->SetDNSTimeout(15000);
			
			//Set the socket timeout
			m_pManager->SetConnectionTimeout(7500);

			//Set it to be multithreaded
			if (bMultithreaded)
				m_pManager->ParseMultithreaded(10);
		}
	}
	ERROR_HANDLER("CRealTimeTraceRouteSocket")
}

CRealTimeTraceRouteSocket::~CRealTimeTraceRouteSocket()
{
	try
	{
		//Delete the CS
		delete m_pCSection;
		delete m_pCSectionResolve;
		delete m_pCSectionGeneral;

		//Delete the manager
		delete m_pManager;
	}
	ERROR_HANDLER("~CRealTimeTraceRouteSocket")
}

BOOL CRealTimeTraceRouteSocket::IsTraceDone()const
{
	return m_bTraceDone;
}

BOOL CRealTimeTraceRouteSocket::TraceRoute(const std::string& rDestinationAddress,
										   DWORD dwTimeout)
{
	try
	{
		return TraceRoute(StringToLong(rDestinationAddress),
						  dwTimeout);
	}
	ERROR_HANDLER_RETURN("TraceRoute",FALSE)
}

BOOL CRealTimeTraceRouteSocket::TraceRoute(IP aDestinationAddress,
										   DWORD dwTimeout)
{
	try
	{
		if (!m_bTraceDone ||
			!m_bInitialized ||
			m_bFinishing ||
			m_bPartialDone)
			return FALSE;

		//Trace not done
		m_bTraceDone=FALSE;
		m_bPartialDone=FALSE;
		m_bFinishing=FALSE;

		//Save the address
		m_aTracedAddress=aDestinationAddress;

		//Clear the data
		for (int iCounter=0;iCounter<=mhMaxHops;++iCounter)
		{
			//Clear the data
			m_aData[iCounter].aReplyingAddress=0;
			m_aData[iCounter].ucTTL=0;
			m_aData[iCounter].dwElapsedTime=0;
			m_aData[iCounter].pDNS=m_pManager;
			m_aData[iCounter].bResolved=FALSE;
			m_aData[iCounter].bTarget=FALSE;
			m_aData[iCounter].bFinal=FALSE;
			m_aData[iCounter].lpUserData=NULL;

			//The CSection
			std::auto_ptr<CGenericCriticalSection> pProtection(COSManager::CreateCriticalSection());

			//Save it
			m_aData[iCounter].pCSection=pProtection;
		}

		m_ucMaxHOP=0;
		m_ucMinHOP=mhMaxHops;

		//Set the last hops data
		//Do we have DNS
		m_aData[mhMaxHops].bTarget=TRUE;

		//No resolvers
		m_ulResolvers=0;

		//We need our port start
		m_usPortStart=GetPortStart();

		//Send the pings
		for (int iPings=0;iPings<3;++iPings)
			for (int iCounter=1;
				 iCounter<=mhMaxHops;
				 ++iCounter)
			{
				//Set the TTL
				SetTTL(iCounter);

				//Set the ID
				SetID(iCounter);

				//Set the sequence
				SetSequence(m_usPortStart);

				//And ping
				if (!Ping(aDestinationAddress,
						  0))
				{
					//Indicate we are done
					m_bTraceDone=TRUE;

					//Call user
					TraceDone(TRUE);

					//Exit
					return FALSE;
				}
			}

		//Set the timeout
		SetTimeout(dwTimeout);

		//Do we have DNS
		if (m_bResolveIPs)
		{
			//Set the resolvers
			AdjustResolvers(1);

			//Create the data
			ResolveThreadData* pThreadData;
			pThreadData=new ResolveThreadData;

			//Populate it
			pThreadData->pClass=this;
			pThreadData->pData=&m_aData[mhMaxHops];

			//DNS lookup for the host
			//Add the reverse DNS
			m_pManager->GetThreadManager()->SubmitJob(ResolveData,(LPVOID)pThreadData);
		}

		//Done
		return TRUE;
	}
	ERROR_HANDLER_RETURN("TraceRoute",FALSE)
}

void CRealTimeTraceRouteSocket::PingDoneData(IP aReplyingAddress,
											 unsigned char ucTTL,
											 DWORD dwTimeElapsed,
											 int iDataSize,
											 unsigned short usID,
											 BOOL bTTLExpired)
{
	try
	{
		//Are we done
		if (m_bTraceDone ||
			m_bPartialDone)
			return;

		if (usID>mhMaxHops)
			return;

		//Do we got it already
		if (m_aData[usID].aReplyingAddress)
			return;

		if (!bTTLExpired)
		{
			//Did we resolve target ?
			if (!m_aData[mhMaxHops].aReplyingAddress)
			{
				{
					//Lock the data
					CCriticalAutoRelease aRelease(m_aData[mhMaxHops].pCSection.get(),TRUE);

					//Put the data
					m_aData[mhMaxHops].aReplyingAddress=aReplyingAddress;
					m_aData[mhMaxHops].dwElapsedTime=dwTimeElapsed;
					m_aData[mhMaxHops].ucTTL=usID;
				}

				//Do we need to send it ?
				if (m_bResolveIPs)
				{
					//Set the resolvers
					AdjustResolvers(1);

					//Create the data
					ResolveThreadData* pThreadData;
					pThreadData=new ResolveThreadData;

					//Populate it
					pThreadData->pClass=this;
					pThreadData->pData=&m_aData[mhMaxHops];

					//Add the reverse DNS
					m_pManager->GetThreadManager()->SubmitJob(ResolveData,(LPVOID)pThreadData);
				}

				//Call it for modify
				TraceDataForModify(&m_aData[mhMaxHops],
								   FALSE);

				//Call user routing
				TraceData(&m_aData[mhMaxHops],
						  FALSE);
			}
		}
		else
		{
			{
				//Lock the data
				CCriticalAutoRelease aRelease(m_pCSection,TRUE);

				//Are we the lowest
				if (usID<m_ucMinHOP)
					m_ucMinHOP=usID;

				//Are we the lowest
				if (usID>m_ucMaxHOP)
					m_ucMaxHOP=usID;
			}

			{
				//Lock the data
				CCriticalAutoRelease aRelease(m_aData[usID].pCSection.get(),TRUE);

				//Populate the data
				m_aData[usID].aReplyingAddress=aReplyingAddress;
				m_aData[usID].dwElapsedTime=dwTimeElapsed;
				m_aData[usID].ucTTL=usID;
				m_aData[usID].pDNS=m_pManager;
			}

			//Do we need to resolve
			if (m_bResolveIPs)
			{
				//Set the resolvers
				AdjustResolvers(1);

				//Create the data
				ResolveThreadData* pThreadData;
				pThreadData=new ResolveThreadData;

				//Populate it
				pThreadData->pClass=this;
				pThreadData->pData=&m_aData[usID];

				//Add the reverse DNS
				m_pManager->GetThreadManager()->SubmitJob(ResolveData,(LPVOID)pThreadData);
			}

			//Call for modify
			TraceDataForModify(&m_aData[usID],
							   FALSE);

			//Call user routing
			TraceData(&m_aData[usID],
					  FALSE);
		}
	}
	ERROR_HANDLER("PingDoneData")
}

BOOL CRealTimeTraceRouteSocket::IsIDOK(unsigned short usID,
									   unsigned short usSequence)const
{
	return m_usPortStart==usSequence &&
		   usID<=mhMaxHops;
}

void CRealTimeTraceRouteSocket::ResolveData(LPVOID lpData)
{
	try
	{
		//Get the data
		std::auto_ptr<ResolveThreadData> pData((ResolveThreadData*)lpData);

		//Our ID
		unsigned short usID;

		//Adjust flag
		BOOL bAdjust;

		//Copy the data
		TraceRouteData aData;

		{
			//Lock the data
			CCriticalAutoRelease aRelease(pData->pData->pCSection.get(),TRUE);

			//Copy it
			aData=*pData->pData;
		}

		//Check the hops 
		if (aData.ucTTL>=mhMaxHops)
		{
			//Adjust the resolvers
			pData->pClass->AdjustResolvers(-1);

			//Dns is done
			pData->pClass->DNSDone();

			//Exit
			return;
		}
		else if (!aData.ucTTL)
			usID=rand()%60000+mhMaxHops;
		else
			usID=aData.ucTTL;

		//Do we need to adjust
		bAdjust=TRUE;

		//Resolve it
		//Do we have the dns
		if (aData.pDNS)
			if (aData.pDNS->GetDNSEntry(aData.aReplyingAddress,
										usID,
										(LPVOID)pData->pData))
				bAdjust=FALSE;
		else
			ReportStaticError(CRealTimeTraceRouteSocket_Class,"ResolveData","Missing DNS information!");

		//Adjust the resolvers
		if (bAdjust)
		{
			//Adjust it
			pData->pClass->AdjustResolvers(-1);

			//Dns is done
			pData->pClass->DNSDone();
		}
	}
	ERROR_HANDLER_STATIC(CRealTimeTraceRouteSocket_Class,"ResolveData")
}

void CRealTimeTraceRouteSocket::TraceData(const TraceRouteData* pData,
										  BOOL bResolved)
{
}

BOOL CRealTimeTraceRouteSocket::IsInitialized()const
{
	return m_bInitialized;
}

BOOL CRealTimeTraceRouteSocket::OnSocketTimeout()
{
	try
	{
		//Kill the timer
		KillTimer();

		//Is the trace done
		if (m_bTraceDone)
			return FALSE;

		//Do we have threads ?
		if (GetResolvers())
		{
			//We are partialy done
			m_bPartialDone=TRUE;

			//Exit
			return FALSE;
		}

		//We are partialy done
		m_bPartialDone=TRUE;

		//Call the trace
		InvokeTraceDone();

		//Done
		return FALSE;
	}
	ERROR_HANDLER_RETURN("OnSocketTimeout",FALSE)
}

void CRealTimeTraceRouteSocket::TraceDone(BOOL bError)
{
}

unsigned char CRealTimeTraceRouteSocket::GetMaxHops()const
{
	return m_ucMaxHOP;
}

const CRealTimeTraceRouteSocket::TraceRouteData* CRealTimeTraceRouteSocket::GetData()const
{
	return (const TraceRouteData*)&m_aData;
}

CRealTimeTraceRouteSocket::CRealTimeDNSManager* CRealTimeTraceRouteSocket::GetDNSManager()const
{
	return m_pManager;
}

void CRealTimeTraceRouteSocket::TraceDataForModify(TraceRouteData* pData,
												   BOOL bResolved)
{
}

void CRealTimeTraceRouteSocket::SetResolveStatus(BOOL bResolve)
{	
	try
	{
		m_bResolveIPs=bResolve;
	}
	ERROR_HANDLER("SetResolveStatus")
}

void CRealTimeTraceRouteSocket::AdjustResolvers(long lAmount)
{
	try
	{
		if (m_bTraceDone)
			return;

		//Lock the data
		CCriticalAutoRelease aRelease(m_pCSectionResolve,TRUE);

		//Adjust it
		m_ulResolvers+=lAmount;
	}
	ERROR_HANDLER("AdjustResolvers")
}

unsigned long CRealTimeTraceRouteSocket::GetResolvers()const
{
	try
	{
		//Lock the data
		CCriticalAutoRelease aRelease(m_pCSectionResolve,TRUE);

		//Adjust it
		return m_ulResolvers;
	}
	ERROR_HANDLER_RETURN("GetResolvers",0)
}

void CRealTimeTraceRouteSocket::InvokeTraceDone()
{
	try
	{
		{
			//Enter the CS
			CCriticalAutoRelease aRelease(m_pCSectionGeneral,TRUE);

			//We are finishing 
			if (m_bFinishing ||
				m_bTraceDone)
				return;
			else
				m_bFinishing=TRUE;
		}

		//Trace flag
		BOOL bTrace;

		{
			//Lock the data
			CCriticalAutoRelease aRelease(m_aData[mhMaxHops].pCSection.get(),TRUE);

			if (!m_aData[mhMaxHops].aReplyingAddress)
			{
				//We need to call user!
				//We are not finishing
				m_bFinishing=FALSE;

				//Exit CS
				aRelease.Exit();

				//Fake last call
				FakeLastRequest();

				//Exit
				return;
			}

			//Set the last hop
			if (m_ucMaxHOP!=mhMaxHops)
				++m_ucMaxHOP;

			//Is it valid ?
			if (m_ucMaxHOP>mhMaxHops)
			{
				//Report it
				ReportError("InvokeTraceDone","Too big max hops!");

				//Set it to normal
				m_ucMaxHOP=mhMaxHops;
			}

			//What is the hop of the target
			m_aData[mhMaxHops].ucTTL=m_ucMaxHOP;
			m_aData[mhMaxHops].bFinal=TRUE;

			//Do we have an IP ?
			if (!m_aData[mhMaxHops].aReplyingAddress)
				m_aData[mhMaxHops].aReplyingAddress=m_aTracedAddress;

			//Do we have a trace
			bTrace=!m_aData[mhMaxHops].sReverseDNS.empty();

			CCriticalAutoRelease aRelease2(m_aData[m_ucMaxHOP].pCSection.get(),TRUE);

			//Save the data to the actual trace
			//Get the string first
			m_aData[m_ucMaxHOP]=m_aData[mhMaxHops];
		}

		//Call user
		TraceDataForModify(&m_aData[m_ucMaxHOP],bTrace);
		TraceData(&m_aData[m_ucMaxHOP],bTrace);

		//Indicate we are done
		m_bTraceDone=TRUE;
		
		//We are not finishing
		m_bFinishing=FALSE;

		//Call handler
		TraceDone(FALSE);
	}
	ERROR_HANDLER("InvokeTraceDone")
}

void CRealTimeTraceRouteSocket::DNSDone()
{
	try
	{
		//Are we partialy done
		if (m_bPartialDone &&
			!m_bTraceDone &&
			!GetResolvers())
			//We are done
			InvokeTraceDone();
	}
	ERROR_HANDLER("DNSDone")
}

unsigned short CRealTimeTraceRouteSocket::GetPortStart()
{
	try
	{
		//Lock the global CS
		CCriticalAutoRelease aRelease(CSpoofBase::GetGlobalCriticalSection(),TRUE);

		//Take the data
		unsigned short usPort;
		usPort=m_sPortStart;

		//Increment it
		++m_sPortStart;

		//Did we exceed ?
		if (m_sPortStart>=65000)
			m_sPortStart=0;

		//Done
		return usPort;
	}
	ERROR_HANDLER_STATIC_RETURN(CRealTimeTraceRouteSocket_Class,"GetPortStart",0)
}

void CRealTimeTraceRouteSocket::FakeLastRequest()
{
	try
	{
		//Did we resolve target ?
		if (!m_aData[mhMaxHops].aReplyingAddress)
		{
			{
				//Lock the data
				CCriticalAutoRelease aRelease(m_aData[mhMaxHops].pCSection.get(),TRUE);

				//Put the data
				m_aData[mhMaxHops].aReplyingAddress=m_aTracedAddress;
				m_aData[mhMaxHops].dwElapsedTime=0;
				m_aData[mhMaxHops].ucTTL=m_ucMaxHOP;
				m_aData[mhMaxHops].pDNS=m_pManager;
			}

			//Do we need to send it ?
			if (m_bResolveIPs)
			{
				//Set the resolvers
				AdjustResolvers(1);

				//Create the data
				ResolveThreadData* pThreadData;
				pThreadData=new ResolveThreadData;

				//Populate it
				pThreadData->pClass=this;
				pThreadData->pData=&m_aData[mhMaxHops];

				//Add the reverse DNS
				m_pManager->GetThreadManager()->SubmitJob(ResolveData,(LPVOID)pThreadData);
			}

			//Call it for modify
			TraceDataForModify(&m_aData[mhMaxHops],
							   FALSE);

			//Call user routing
			TraceData(&m_aData[mhMaxHops],
					  FALSE);
		}
	}
	ERROR_HANDLER("FakeLastRequest")
}

KOMODIA_NAMESPACE_END