/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#include "stdafx.h"
#include "TCPPortScannerLinear.h"

#include "ErrorHandlerMacros.h"

#ifdef _MEMORY_DEBUG 
	#define new	   DEBUG_NEW  
	#define malloc DEBUG_MALLOC  
    static char THIS_FILE[] = __FILE__;  
#endif

KOMODIA_NAMESPACE_START

#define CTCPPortScannerLinear_Class "CTCPPortScannerLinear"

CTCPPortScannerLinear::CTCPPortScannerLinear() : CTCPPortScanner()
{
	try
	{
		//Set our name
		SetName(CTCPPortScannerLinear_Class);
	}
	ERROR_HANDLER("CTCPPortScannerLinear")
}

CTCPPortScannerLinear::~CTCPPortScannerLinear()
{
}

BOOL CTCPPortScannerLinear::Scan(const std::string& rDestinationAddress)
{
	try
	{
		return Scan(CSpoofBase::StringToLong(rDestinationAddress));
	}
	ERROR_HANDLER_RETURN("Scan",FALSE)
}

BOOL CTCPPortScannerLinear::Scan(IP aTarget)
{
	//Check max sockets are ok
	if (GetMaxSockets()<=0 ||
		!IsDone() ||
		!aTarget)
	{
		//Report it
		ReportError("Scan","Invalid parameters!");

		//Exit
		return FALSE;
	}

	try
	{
		//Reset the pair
		ResetPair();

		//Can we scan
		if (!CanScan())
		{
			//Error
			ReportError("Scan","No ports given!");

			//Error
			SetError(TRUE);

			//Exit
			return FALSE;
		}

		//Set the flags
		SetError(FALSE);
		SetDone(FALSE);

		//Save the number of max sockets
		ResetSocketCount();

		//Delete the port data
		DeletePortsMap();

		//Delete other data
		ResetData();

		//Set our target
		SetTarget(aTarget);

		//Start the loop
		for (int iCounter=0;
			 iCounter<GetMaxSockets();
			 ++iCounter)
		{
			BOOL bQuit;
			bQuit=FALSE;

			//Try to allocate sockets
			while (!bQuit)
			{
				//Another socket
				AdjustNumberOfSockets(1);

				//Create it
				if (!(bQuit=NewSocket()))
					ReportError("Scan","Failed to create socket!");
			}
		}

		//Done
		return TRUE;
	}
	ERROR_HANDLER_RETURN("Scan",FALSE)
}

KOMODIA_NAMESPACE_END
