/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#if !defined(AFX_TCPPORTSCANNERSTEALTH_H__25CD2089_2AAE_4914_80CF_4ECD1B9DCD9E__INCLUDED_)
#define AFX_TCPPORTSCANNERSTEALTH_H__25CD2089_2AAE_4914_80CF_4ECD1B9DCD9E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "PeriodicThread.h"

#include "SniffSocket.h"

#include "TCPPortScanner.h"

#include <map>

KOMODIA_NAMESPACE_START

class CGenericCriticalSection;

class CTCPPortScannerStealth : public CTCPPortScanner  
{
public:
	//Do the scan
	virtual BOOL Scan(const std::string& rDestinationAddress);
	virtual BOOL Scan(IP aTarget);

	//Ctor and Dtor
	CTCPPortScannerStealth(IP aBindInterface);
	CTCPPortScannerStealth(const std::string& rBindInterface);
	virtual ~CTCPPortScannerStealth();
protected:
	//Actual a new socket
	//Derived class should override this for their sockets
	virtual CTCPSocketAsync* AllocateSocket(unsigned short usPort)const;

	//Unallocate the socket
	virtual void DestroySocket(CTCPSocketAsync* pSocket)const;

	//Scan is done (for library)
	virtual void TCPScanDoneLib(BOOL bError);

	//Port was scanned (no resolution yet, for lib to overide)
	virtual void PortScanned(unsigned short usPort);

	//Remove a port from the list
	void RemovePort(unsigned short usPort);

	//Get the bind interface
	IP GetBindInterface()const;

	//Reset ports data
	void ResetPortsData();

	//Start the timeout thread
	void StartThread(BOOL bDefaultDisconnected=TRUE);
private:
	//Our TCP socket
	class CTCPScanner : public CTCPSocketAsync
	{
	public:
		//Our async connection
		virtual BOOL Connect(unsigned short usSourcePort,
							 IP aDestinationAddress,
							 unsigned short usDestinationPort,
							 BOOL bDisableAsync=FALSE,
							 BOOL bForceErrorEvent=FALSE)
		{
			return CTCPSocketAsync::Connect(2000,
											aDestinationAddress,
											usDestinationPort,
											bDisableAsync,
											bForceErrorEvent);
		}

		//Ctor and Dtor
		CTCPScanner() : CTCPSocketAsync(TRUE)
		{
		}
		virtual ~CTCPScanner()
		{
		}
	protected:
		//No events
		NO_OnSocketAccept
		NO_OnSocketOOB
		NO_OnSocketWrite
		NO_OnSocketTimeout
		NO_OnSocketConnect
		NO_OnSocketClose
		NO_OnSocketReceive
	};
private:
	//Our sniffing socket
	class CSniffingSocket : public CSniffSocket
	{
	public:
		//Stop sniffing
		void Stop();

		//Ctor and Dtor
		CSniffingSocket(CTCPPortScannerStealth* pScanner);
		virtual ~CSniffingSocket();
	protected:
		//Our sniff handler
		virtual BOOL OnSocketReceive(int iErrorCode);
	private:
		//Analyze the TCP packet
		void AnalyzeTCP(const char* pTCPBuffer);

		//Our father
		CTCPPortScannerStealth* m_pFather;

		//Are we stopped
		BOOL m_bStop;
	};

	friend CSniffingSocket;
private:
	//Our map of ports
	typedef std::map<unsigned short,unsigned long> PortsMap;
private:
	//No copy Ctor
	CTCPPortScannerStealth(const CTCPPortScannerStealth& rStealth);

	//No copy operator
	CTCPPortScannerStealth& operator=(const CTCPPortScannerStealth& rStealth);

	//Create the sniffer socket
	BOOL CreateSniffer();

	//Our periodic thread
	static BOOL TimeoutProc(CPeriodicThread::ThreadStage aStage,
							LPVOID pParam);

	//Our scanner socket
	CTCPScanner* m_pScanner;

	//Our bind interface
	IP m_aBindInterface;

	//Our sniffer socket
	CSniffingSocket* m_pSniffer;

	//Our CS
	CGenericCriticalSection* m_pCSection;

	//Our ports map
	PortsMap m_aPortsMap;

	//Our periodic timeout thread
	CPeriodicThread* m_pThread;

	//Default mode
	BOOL m_bDefaultDisconnected;
};

KOMODIA_NAMESPACE_END

#endif // !defined(AFX_TCPPORTSCANNERSTEALTH_H__25CD2089_2AAE_4914_80CF_4ECD1B9DCD9E__INCLUDED_)
