/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#include "stdafx.h"
#include "TraceRouteSocket.h"

#include "ErrorHandlerMacros.h"

#ifdef _MEMORY_DEBUG 
	#define new	   DEBUG_NEW  
	#define malloc DEBUG_MALLOC  
    static char THIS_FILE[] = __FILE__;  
#endif

KOMODIA_NAMESPACE_START

#define CTraceRouteSocket_Class "CTraceRouteSocket"

CTraceRouteSocket::CTraceRouteSocket() : CPingSocket(),
										 m_bTraceDone(TRUE)
{
	try
	{
		//Set our name
		SetName(CTraceRouteSocket_Class);
	}
	ERROR_HANDLER("CTraceRouteSocket")
}

CTraceRouteSocket::~CTraceRouteSocket()
{
}

BOOL CTraceRouteSocket::IsTraceDone()const
{
	return m_bTraceDone;
}

BOOL CTraceRouteSocket::TraceRoute(const std::string& rDestinationAddress,
								   DWORD dwTimeout,
								   unsigned char ucTTL)
{
	try
	{
		return TraceRoute(StringToLong(rDestinationAddress),
						  dwTimeout,
						  ucTTL);
	}
	ERROR_HANDLER_RETURN("TraceRoute",FALSE)
}

BOOL CTraceRouteSocket::TraceRoute(IP aDestinationAddress,
								   DWORD dwTimeout,
								   unsigned char ucTTL)
{
	try
	{
		if (!m_bTraceDone)
			return FALSE;

		//Starting TTL
		m_ucTTL=1;

		//Max TLL
		m_ucMaxTTL=ucTTL;

		//Trace not done
		m_bTraceDone=FALSE;

		//Clear the data
		m_aData.clear();

		//Save the timeout
		m_dwTimeout=dwTimeout;

		//Save the address
		m_aFinalAddress=aDestinationAddress;

		//And ping
		return Ping(aDestinationAddress,
					dwTimeout);
	}
	ERROR_HANDLER_RETURN("TraceRoute",FALSE)
}

CTraceRouteSocket::TraceRouteVector& CTraceRouteSocket::GetData()
{
	return m_aData;
}

void CTraceRouteSocket::TakeData(TraceRouteVector& rVector)
{
	m_aData.swap(rVector);
}

BOOL CTraceRouteSocket::Ping(IP aDestinationAddress,
							 DWORD dwTimeout)
{
	try
	{
		//Set our TTL
		SetTTL(m_ucTTL);

		//And original ping
		return CPingSocket::Ping(aDestinationAddress,
								 dwTimeout);
	}
	ERROR_HANDLER_RETURN("Ping",FALSE)
}

void CTraceRouteSocket::PingDone(BOOL bTimeout)
{
	try
	{
		//The data record
		TraceRouteData aData;
		aData.ucTTL=m_ucTTL;
		aData.bTimeout=bTimeout;
		aData.dwElapsedTime=GetTimeElapsed();

		//Is it a timeout ?
		if (bTimeout)
			//Can we ping again
			if (m_ucTTL>m_ucMaxTTL)
				m_bTraceDone=TRUE;
			else
				;
		else
		{
			//Save the data
			aData.aReplyingAddress=GetReplyingAddress();

			//Are we done
			if (aData.aReplyingAddress==m_aFinalAddress)
				m_bTraceDone=TRUE;
		}

		//Add the data
		m_aData.push_back(aData);

		//Call user with the data
		TraceData(aData);

		//Check if we are done ?
		if (m_bTraceDone ||
			(bTimeout && 
			 m_ucTTL>m_ucMaxTTL))
			//Dispatch the event
			TraceDone();
		else
		{
			//Increase our ttl
			++m_ucTTL;

			//Send the next ping
			if (!Ping(m_aFinalAddress,
					  m_dwTimeout))
			{
				//There's an error report and continue
				ReportError("PingDone","Failed to ping!");

				//Make it a timeout
				PingDone(TRUE);
			}
		}
	}
	ERROR_HANDLER("PingDone")
}

void CTraceRouteSocket::TraceDone()
{
}

void CTraceRouteSocket::TraceData(const TraceRouteData& rData)
{
}

KOMODIA_NAMESPACE_END
