/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#if !defined(AFX_TRACEROUTESOCKET_H__8C25D77A_4D1D_4678_88EC_EF4AB3177E79__INCLUDED_)
#define AFX_TRACEROUTESOCKET_H__8C25D77A_4D1D_4678_88EC_EF4AB3177E79__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "PingSocket.h"

#include <vector>

KOMODIA_NAMESPACE_START

class CTraceRouteSocket : public CPingSocket  
{
public:
	//Our data sturct
	typedef struct _TraceRouteData
	{
		IP				aReplyingAddress;
		unsigned char	ucTTL;
		DWORD			dwElapsedTime;
		BOOL			bTimeout;
		LPVOID			lpUserData;
	} TraceRouteData;

	//The data we pass back
	typedef std::vector<TraceRouteData> TraceRouteVector;
public:
	//Take the data
	void TakeData(TraceRouteVector& rVector);

	//Get the data
	TraceRouteVector& GetData();

	//Is the traced done
	virtual BOOL IsTraceDone()const;

	//Do a trace
	virtual BOOL TraceRoute(const std::string& rDestinationAddress,
							DWORD dwTimeout,
							unsigned char ucTTL=30);

	virtual BOOL TraceRoute(IP aDestinationAddress,
							DWORD dwTimeout,
							unsigned char ucTTL=30);

	//Ctor and Dtor
	CTraceRouteSocket();
	virtual ~CTraceRouteSocket();
protected:
	//Traceroute implementation of ping
	virtual BOOL Ping(IP aDestinationAddress,
					  DWORD dwTimeout=3000);

	//Events for user to overide
	virtual void PingDone(BOOL bTimeout);

	//Event for user to overide
	virtual void TraceDone();

	//For user to overide, per TTL data
	virtual void TraceData(const TraceRouteData& rData);
private:
	//No copy Ctor
	CTraceRouteSocket(const CTraceRouteSocket& rSocket);

	//No copy operator
	CTraceRouteSocket& operator=(const CTraceRouteSocket& rSocket);

	//Is the trace done ?
	BOOL m_bTraceDone;

	//Current TTL
	unsigned char m_ucTTL;

	//Maximum TTL
	unsigned char m_ucMaxTTL;

	//Our vector
	TraceRouteVector m_aData;

	//Our timeout
	DWORD m_dwTimeout;

	//The address we are routing
	IP m_aFinalAddress;
};

KOMODIA_NAMESPACE_END

#endif // !defined(AFX_TRACEROUTESOCKET_H__8C25D77A_4D1D_4678_88EC_EF4AB3177E79__INCLUDED_)
