/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#include "stdafx.h"
#include "TraceRouteSocketReverseDNS.h"

#include "ErrorHandlerMacros.h"
#include "OSManager.h"
#include "GenericThread.h"

#include "DNSManager.h"

#ifdef _MEMORY_DEBUG 
	#define new	   DEBUG_NEW  
	#define malloc DEBUG_MALLOC  
    static char THIS_FILE[] = __FILE__;  
#endif

KOMODIA_NAMESPACE_START

#define CTraceRouteSocketReverseDNS_Class "CTraceRouteSocketReverseDNS"

CTraceRouteSocketReverseDNS::CTraceRouteSocketReverseDNS(std::string sDNSServer,
														 BOOL bDNSTcp,
														 BOOL bAsync) : CTraceRouteSocket(),
																		m_bAsync(bAsync),
																		m_pThread(NULL),
																		m_pDNSManager(NULL),
																		m_bTraceDone(TRUE),
																		m_bInitialized(FALSE)
{	
	try
	{
		//Set our name
		SetName(CTraceRouteSocketReverseDNS_Class);

		//Create the manager
		m_pDNSManager=new KomodiaDNS::CDNSManager(sDNSServer,
												  bDNSTcp,
												  bAsync);

		//Initialize it
		if (!(m_bInitialized=m_pDNSManager->Initialize()))
			ReportError("CTraceRouteSocketReverseDNS","Failed to initialize sockets!");
	}
	ERROR_HANDLER("CTraceRouteSocketReverseDNS")
}

CTraceRouteSocketReverseDNS::CTraceRouteSocketReverseDNS(const std::string& rDNSServer,
														 BOOL bDNSTcp) : CTraceRouteSocket(),
																		 m_bAsync(FALSE),
																		 m_pThread(NULL),
																		 m_pDNSManager(NULL),
																		 m_bTraceDone(TRUE),
																		 m_bInitialized(FALSE)
{	
	try
	{
		//Set our name
		SetName(CTraceRouteSocketReverseDNS_Class);

		//Create the manager
		m_pDNSManager=new KomodiaDNS::CDNSManager(rDNSServer,
												  bDNSTcp,
												  m_bAsync);

		//Initialize it
		if (!(m_bInitialized=m_pDNSManager->Initialize()))
			ReportError("CTraceRouteSocketReverseDNS","Failed to initialize sockets!");
	}
	ERROR_HANDLER("CTraceRouteSocketReverseDNS")
}

CTraceRouteSocketReverseDNS::~CTraceRouteSocketReverseDNS()
{
	try
	{
		//Delete the thread
		delete m_pThread;

		//Delete the manager
		delete m_pDNSManager;
	}
	ERROR_HANDLER("~CTraceRouteSocketReverseDNS")
}

void CTraceRouteSocketReverseDNS::TraceDone()
{
	try
	{
		if (m_pThread)
			delete m_pThread;

		//Spawn the thread
		m_pThread=COSManager::CreateThread(DNSResolving);
		m_pThread->Start((LPVOID)this);
	}
	ERROR_HANDLER("CTraceRouteSocketReverseDNS")
}

void CTraceRouteSocketReverseDNS::TraceData(const TraceRouteData& rData)
{
}

DWORD CTraceRouteSocketReverseDNS::DNSResolving(LPVOID lpData)
{
	try
	{
		//Get the class
		CTraceRouteSocketReverseDNS* pClass;
		pClass=(CTraceRouteSocketReverseDNS*)lpData;

		//Start iterating the data
		TraceRouteVector aData;
		pClass->TakeData(aData);

		CTraceRouteSocket::TraceRouteVector::const_iterator aIterator=aData.begin();

		//Iterate it
		while (aIterator!=aData.end())
		{
			//Save the data
			DNSTraceRouteData aDNSData;
			aDNSData.aReplyingAddress=aIterator->aReplyingAddress;
			aDNSData.bTimeout=aIterator->bTimeout;
			aDNSData.dwElapsedTime=aIterator->dwElapsedTime;
			aDNSData.ucTTL=aIterator->ucTTL;

			//Is it a timeout
			if (!aIterator->bTimeout)
			{
				//Query the DNS
				if (!pClass->m_pDNSManager->GetDNSEntry(aDNSData.aReplyingAddress))
					ReportStaticError(CTraceRouteSocketReverseDNS_Class,"DNSResolving","Failed to query DNS!");
				else
				{
					//Get the DNS answer
					KomodiaDNS::CDNSAnswers* pAnswers;
					pAnswers=pClass->m_pDNSManager->Receive();

					//Do we have it
					if (!pAnswers)
						ReportStaticError(CTraceRouteSocketReverseDNS_Class,"DNSResolving","Failed to query DNS!");
					else if (pAnswers->IsResolved())
						aDNSData.sReverseDNS=pAnswers->GetAnswer();
				}
			}

			//Add the data
			pClass->m_aData.push_back(aDNSData);

			//Increase the iterator
			++aIterator;
		}

		//Indicate trace is done
		pClass->m_bTraceDone=TRUE;

		//Call the trace is done
		pClass->DNSTraceDone();

		//Done
		return TRUE;
	}
	ERROR_HANDLER_STATIC_RETURN(CTraceRouteSocketReverseDNS_Class,"DNSResolving",THREAD_DO_NOTHING_EXIT_VALUE)
}

void CTraceRouteSocketReverseDNS::DNSTraceDone()
{
}

BOOL CTraceRouteSocketReverseDNS::IsTraceDone()const
{
	return m_bTraceDone;
}

void CTraceRouteSocketReverseDNS::TakeDNSData(DNSTraceRouteVector& rVector)
{
	try
	{
		m_aData.swap(rVector);
	}
	ERROR_HANDLER("TakeDNSData")
}

CTraceRouteSocketReverseDNS::DNSTraceRouteVector& CTraceRouteSocketReverseDNS::GetDNSData()
{
	return m_aData;
}

BOOL CTraceRouteSocketReverseDNS::TraceRoute(IP aDestinationAddress,
											 DWORD dwTimeout,
											 unsigned char ucTTL)
{
	try
	{
		if (!m_bInitialized)
			return FALSE;

		//Are we tracing ?
		if (!m_bTraceDone)
			return FALSE;

		//Clear the data
		m_bTraceDone=FALSE;

		//The vector
		m_aData.clear();

		//And start
		return CTraceRouteSocket::TraceRoute(aDestinationAddress,
											 dwTimeout,
											 ucTTL);
	}
	ERROR_HANDLER_RETURN("TraceRoute",FALSE)
}

BOOL CTraceRouteSocketReverseDNS::IsInitialized()const
{
	return m_bInitialized;
}

KOMODIA_NAMESPACE_END
