/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#if !defined(AFX_UDPSCANNER_H__68F910AA_B996_41E7_8565_1923981A9B54__INCLUDED_)
#define AFX_UDPSCANNER_H__68F910AA_B996_41E7_8565_1923981A9B54__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "ICMPSocketAsync.h"

#include "TCPPortScannerStealth.h"

KOMODIA_NAMESPACE_START

class CUDPSocket;

class CUDPScanner : public CTCPPortScannerStealth  
{
public:
	//Do the scan
	virtual BOOL Scan(const std::string& rDestinationAddress);
	virtual BOOL Scan(IP aTarget);

	//Ctor and Dtor
	CUDPScanner(IP aBindInterface);
	CUDPScanner(const std::string& rBindInterface);
	virtual ~CUDPScanner();
protected:
	//Actual a new socket
	//Derived class should override this for their sockets
	virtual CTCPSocketAsync* AllocateSocket(unsigned short usPort)const;

	//Unallocate the socket
	virtual void DestroySocket(CTCPSocketAsync* pSocket)const;

	//Scan is done (for library)
	virtual void TCPScanDoneLib(BOOL bError);
private:
	//Our TCP socket, that encapsulates the UDP socket
	class CTCPScanner : public CTCPSocketAsync
	{
	public:
		//Overide the connect
		virtual BOOL Connect(unsigned short usSourcePort,
							 IP aDestinationAddress,
							 unsigned short usDestinationPort,
							 BOOL bDisableAsync=FALSE,
							 BOOL bForceErrorEvent=FALSE);

		//Create the socket
		virtual BOOL Create();

		//Ctor and Dtor
		CTCPScanner();
		virtual ~CTCPScanner();
	protected:
		//No events
		NO_OnSocketAccept
		NO_OnSocketOOB
		NO_OnSocketWrite
		NO_OnSocketTimeout
		NO_OnSocketConnect
		NO_OnSocketClose
		NO_OnSocketReceive
	private:
		//Our UDP socket
		CUDPSocket* m_pSocket;
	};
private:
	//Our sniffing socket
	class CSniffingSocket : public CICMPSocketAsync
	{
	public:
		//Ctor and Dtor
		CSniffingSocket(CUDPScanner* pScanner);
		virtual ~CSniffingSocket();
	protected:
		//No events
		NO_OnSocketAccept
		NO_OnSocketOOB
		NO_OnSocketWrite
		NO_OnSocketTimeout
		NO_OnSocketConnect
		NO_OnSocketClose

		//Our sniff handler
		virtual BOOL OnSocketReceive(int iErrorCode);
	private:
		//Our father
		CUDPScanner* m_pFather;
	};

	friend CSniffingSocket;
private:
	//No copy Ctor
	CUDPScanner(const CUDPScanner& rScanner);

	//No copy operator
	CUDPScanner& operator=(const CUDPScanner& rScanner);

	//Create the sniffer socket
	BOOL CreateSniffer();

	//Our socket
	CTCPScanner* m_pScanner;

	//Our sniffer socket
	CSniffingSocket* m_pSniffer;
};

KOMODIA_NAMESPACE_END

#endif // !defined(AFX_UDPSCANNER_H__68F910AA_B996_41E7_8565_1923981A9B54__INCLUDED_)
