/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#if !defined(AFX_MANAGEDTHREAD_H__4797BEBC_129B_4971_940F_40205285990F__INCLUDED_)
#define AFX_MANAGEDTHREAD_H__4797BEBC_129B_4971_940F_40205285990F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "ErrorHandler.h"
#include "GenericThread.h"

KOMODIA_NAMESPACE_START

class CGenericEvent;

class CManagedThread : public CErrorHandler
{
public:
	//Our thread proc
	typedef DWORD (*ManagedThreadProc)(CGenericEvent* pExitEvent,
									   LPVOID pParam);
	typedef ManagedThreadProc LPManagedThreadProc;
public:
	//Set the thread priority
	void SetPriority(CGenericThread::ThreadPriority aPriority);

	//Get the thread we are using
	CGenericThread* GetThread()const;

	//Set the thread timeout
	void SetThreadTimeout(DWORD dwTimeout);

	//Start the thread
	//lpData - Custome user data to pass to the thread
	BOOL Start(LPVOID lpData);

	//Stop the thread
	BOOL Stop();

	//Ctor and Dtor
	CManagedThread(LPManagedThreadProc pProc);
	virtual ~CManagedThread();
private:
	//No copy Ctor
	CManagedThread(const CManagedThread& rThread);

	//No assignment operator
	CManagedThread& operator=(const CManagedThread& rThread);

	//Our thread proc
	static DWORD ManagedThread(LPVOID lpData);

	//Our thread
	CGenericThread* m_pThread;

	//The user proc
	LPManagedThreadProc m_pUserProc;

	//Our enter event
	CGenericEvent* m_pEnterEvent;

	//Our exit event
	CGenericEvent* m_pExitEvent;

	//Thread timeout (for events)
	DWORD m_dwThreadTimeout;

	//Are we in the destructor
	BOOL m_bDestructor;
	
	//User's data
	LPVOID m_lpData;
};

KOMODIA_NAMESPACE_END

#endif // !defined(AFX_MANAGEDTHREAD_H__4797BEBC_129B_4971_940F_40205285990F__INCLUDED_)
