/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#include "stdafx.h"
#include "SniffSocket.h"

#include "ErrorHandlerMacros.h"

#ifdef _MEMORY_DEBUG 
	#define new	   DEBUG_NEW  
	#define malloc DEBUG_MALLOC  
    static char THIS_FILE[] = __FILE__;  
#endif

KOMODIA_NAMESPACE_START

#define CSniffSocket_Class "CSniffSocket"

CSniffSocket::CSniffSocket() : CSpoofSocket(TRUE), 
							   CAsyncSocket()
{
	try
	{
		//Set our name
		SetName(CSniffSocket_Class);
	}
	ERROR_HANDLER("CSniffSocket")
}

CSniffSocket::~CSniffSocket()
{
	try
	{
		RemoveSocketFromList();
	}
	ERROR_HANDLER("~CSniffSocket")
}

BOOL CSniffSocket::Sniff(BOOL bSniff)
{
	try
	{
		//Try to move to sniff mode
		if (CSpoofSocket::Sniff(bSniff))
			return !WSAAsyncSelect(GetHandle(),
								   GetWindowHandle(),
								   WM_SOCKET_GENERAL,
								   FD_READ);
		else
			return FALSE;
	}
	ERROR_HANDLER_RETURN("Sniff",FALSE)
}
	
BOOL CSniffSocket::Create()
{
	try
	{
		//Try to create as regular IP socket
		SetProtocol(IPPROTO_IP);
		if (CSpoofSocket::Create(IPPROTO_IP))
		{
			//OK, we can use this socket
			AddSocketToList();

			//Done
			return TRUE;
		}
		else
			return FALSE;
	}
	ERROR_HANDLER_RETURN("Create",FALSE)
}

SOCKET CSniffSocket::GetAsyncHandle() const
{
	return GetHandle();
}

BOOL CSniffSocket::SetAsync()
{
	//Do nothing
	return TRUE;
}

KOMODIA_NAMESPACE_END
