/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#include "stdafx.h"
#include "TCPCrafter.h"

#include "ErrorHandlerMacros.h"
#include "Array_ptr.h"

#include <memory>

#ifdef _MEMORY_DEBUG 
	#define new	   DEBUG_NEW  
	#define malloc DEBUG_MALLOC  
    static char THIS_FILE[] = __FILE__;  
#endif

KOMODIA_NAMESPACE_START

#define CTCPCrafter_Class "CTCPCrafter"

CTCPCrafter::CTCPCrafter() : CTCPSocket(TRUE), 
							 CIPCrafter()
{
	try
	{
		//Set our name
		CTCPSocket::SetName(CTCPCrafter_Class);

		//Set our default
		m_bTCPUseDefaultChecksum=TRUE;

		//No sequence
		m_uiSequenceNumber=0;

		//No ack
		m_uiAcknowledgeNumber=0;

		//No offset
		m_ucDataOffset=TCPHeaderLength/4;

		//No flags
		m_ucFlags=0;

		//No window
		m_usWindow=512;

		//No urgent pointer
		m_usUrgentPointer=0;

		//Set the protocol
		SetProtocol(IPPROTO_TCP);
	}
	ERROR_HANDLER("CTCPCrafter")
}

CTCPCrafter::~CTCPCrafter()
{
}

LPIpHeader CTCPCrafter::ConstructIPHeader(unsigned char  ucProtocol,
										  unsigned short usFragmentationFlags,
										  unsigned char  ucTTL,
										  unsigned short usIdentification,
										  unsigned char  ucHeaderLength)const
{
	return ConstructCraftedIPHeader(ucProtocol,
									usFragmentationFlags,
									ucTTL,
									usIdentification,
									ucHeaderLength);
}

void CTCPCrafter::FinalIPHeader(LPIpHeader lpHead)const
{
	try
	{
		//Check if we need to modify the header
		if (!m_DefaultChecksum)
			lpHead->usChecksum=htons(m_usChecksum);
	}
	ERROR_HANDLER("FinalIPHeader")
}

void CTCPCrafter::SetSequenceNumber(unsigned int uiSequenceNumber)
{
	m_uiSequenceNumber=uiSequenceNumber;
}

void CTCPCrafter::SetAcknowledgeNumber(unsigned int uiAcknowledgeNumber)
{
	m_uiAcknowledgeNumber=uiAcknowledgeNumber;
}

void CTCPCrafter::SetDataOffset(unsigned char ucDataOffset)
{
	m_ucDataOffset=ucDataOffset;
}

void CTCPCrafter::SetFlags(unsigned char ucFlags)
{
	m_ucFlags=ucFlags;
}

void CTCPCrafter::SetWindow(unsigned short usWindow)
{
	m_usWindow=usWindow;
}

void CTCPCrafter::SetTCPChecksum(unsigned short usTCPChecksum)
{
	m_usTCPChecksum=usTCPChecksum;
}

void CTCPCrafter::SetTCPUseDefaultChecksum(BOOL bDefault)
{
	m_bTCPUseDefaultChecksum=bDefault;
}

void CTCPCrafter::SetUrgentPointer(unsigned short usUrgentPointer)
{
	m_usUrgentPointer=usUrgentPointer;
}

LPTCPHeader CTCPCrafter::ConstructTCPHeader(unsigned short usSourcePort,
										    unsigned short usDestinationPort,
										    unsigned char ucHeaderLength)const
{
	try
	{
		//Create the initial header
		LPTCPHeader lpHead;
		lpHead=CTCPSocket::ConstructTCPHeader(usSourcePort,
											  usDestinationPort,
											  ucHeaderLength);

		if (!lpHead)
		{
			//Report it
			ReportError("ConstructTCPHeader","Received null TCP header!");

			//Exit
			return NULL;
		}

		//Overide some data
		lpHead->usWindows=htons(m_usWindow);

		//Set the packet number
		lpHead->ulAcknowledgeNumber=htonl(m_uiAcknowledgeNumber);

		//And the sequence
		lpHead->ulSequenceNumber=htonl(m_uiSequenceNumber);

		//Data offset
		lpHead->ucDataOffset=m_ucDataOffset << 4;

		//Flags
		lpHead->ucFlags=m_ucFlags;

		//Urgent pointer
		lpHead->usUrgentPointer=htons(m_usUrgentPointer);

		//Return it to the user
		return lpHead;
	}
	ERROR_HANDLER_RETURN("ConstructTCPHeader",NULL)
}

void CTCPCrafter::FinalTCPHeader(LPTCPHeader lpHead)const
{
	try
	{
		if (!m_bTCPUseDefaultChecksum)
			lpHead->usChecksum=htons(m_usTCPChecksum);
	}
	ERROR_HANDLER("FinalTCPHeader")
}

BOOL CTCPCrafter::SendRaw(unsigned short usSourcePort, 
						  const std::string& rDestinationAddress, 
						  unsigned short usDestinationPort,
						  const char* pBuffer,
						  unsigned long ulBufLength,
						  unsigned char ucFlags)
{
	try
	{
		return CTCPSocket::SendRaw(usSourcePort,
								   rDestinationAddress,
								   usDestinationPort,
								   pBuffer,
								   ulBufLength,
								   ucFlags);
	}
	ERROR_HANDLER_RETURN("SendRaw",FALSE)
}

BOOL CTCPCrafter::SendRaw(unsigned short usSourcePort, 
						  IP aDestinationAddress, 
						  unsigned short usDestinationPort,
						  const char* pBuffer,
						  unsigned long ulBufLength,
						  unsigned char ucFlags)
{
	try
	{
		//Our TCP flags
		unsigned char uclFlags;
		
		if (!ucFlags)
			uclFlags=m_ucFlags;
		else
			uclFlags=ucFlags;

		return CTCPSocket::SendRaw(usSourcePort,
								   aDestinationAddress,
								   usDestinationPort,
								   pBuffer,
								   ulBufLength,
								   uclFlags);
	}
	ERROR_HANDLER_RETURN("SendRaw",FALSE)
}

char* CTCPCrafter::GetCraftedPacket(unsigned short usSourcePort,
									unsigned long ulDestinationAddress,
									const char* pBuffer,
									unsigned long ulBufferSize,
									unsigned short usDestinationPort,
									unsigned short& usTotalSize)const
{
	try
	{
		//Let's try our first attack
		LPTCPHeader lpHead;

		//Header length
		int iHeaderLength;
		iHeaderLength=TCPHeaderLength;

		//Create the header
		lpHead=ConstructTCPHeader(usSourcePort,
								  usDestinationPort,
								  iHeaderLength);

		//Protect it
		std::auto_ptr<TCPHeader> pProtection(lpHead);

		//Our total length
		int iTotalLength;
		iTotalLength=iHeaderLength+ulBufferSize;

		//Allocate the buffer
		char* pNewBuffer;
		pNewBuffer=new char[iTotalLength];

		//Protect it
		CArray_ptr<char> pBufferProtection(pNewBuffer);

		//Copy the header
		memcpy(pNewBuffer,
			   lpHead,
			   iHeaderLength);

		//Copy the data
		if (ulBufferSize)
			//Copy the data
			memcpy(pNewBuffer+iHeaderLength,
				   pBuffer,
				   ulBufferSize);

		//Calculate the checksum
		lpHead->usChecksum=CalculatePseudoChecksum(pNewBuffer,
												   iTotalLength,
												   ulDestinationAddress,
												   iTotalLength);

		//Last change to the header
		FinalTCPHeader(lpHead);

		//Copy the header
		memcpy(pNewBuffer,
			   lpHead,
			   iHeaderLength);

		//Send the data
		char* pNewData;
		pNewData=CIPCrafter::GetCraftedPacket(this,
											  ulDestinationAddress,
											  pNewBuffer,
											  iTotalLength,
											  usTotalSize);

		//Exit
		return pNewData;
	}
	ERROR_HANDLER_RETURN("GetCraftedPacket",NULL)
}

KOMODIA_NAMESPACE_END