/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#if !defined(AFX_TCPCRAFTER_H__E2C01EE5_8ABA_4D74_91D6_E80D1D404E5C__INCLUDED_)
#define AFX_TCPCRAFTER_H__E2C01EE5_8ABA_4D74_91D6_E80D1D404E5C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "TCPSocket.h"
#include "IPCrafter.h"

KOMODIA_NAMESPACE_START

class CTCPCrafter : public CTCPSocket,
					public CIPCrafter
{
public:
	//Get the memory of a crafted packet (returns an allocated packet)
	char* GetCraftedPacket(unsigned short usSourcePort,
						   unsigned long ulDestinationAddress,
						   const char* pBuffer,
						   unsigned long ulBufferSize,
						   unsigned short usDestinationPort,
						   unsigned short& usTotalSize)const;

	//Set urgent pointer
	void SetUrgentPointer(unsigned short usUrgentPointer);

	//Indicate if we want to use TCP default checksum
	void SetTCPUseDefaultChecksum(BOOL bDefault);

	//Set the TCP checksum
	void SetTCPChecksum(unsigned short usTCPChecksum);

	//Set the window size
	void SetWindow(unsigned short usWindow);

	//Set the TCP flags
	void SetFlags(unsigned char ucFlags);

	//Set the data offset (TCP)
	void SetDataOffset(unsigned char ucDataOffset);

	//Set the acknowledge number
	void SetAcknowledgeNumber(unsigned int uiAcknowledgeNumber);

	//Set the sequence number
	void SetSequenceNumber(unsigned int uiSequenceNumber);

	//Send raw packet
	//Positive - The number of bytes received.
	//Zero - Socket has been closed.
	//Negative - Error
	virtual BOOL SendRaw(unsigned short usSourcePort, 
						 IP aDestinationAddress, 
						 unsigned short usDestinationPort,
						 const char* pBuffer,
						 unsigned long ulBufLength,
						 unsigned char ucFlags=0);

	//Send raw packet
	//Positive - The number of bytes received.
	//Zero - Socket has been closed.
	//Negative - Error
	virtual BOOL SendRaw(unsigned short usSourcePort, 
						 const std::string& rDestinationAddress,
						 unsigned short usDestinationPort,
						 const char* pBuffer,
						 unsigned long ulBufLength,
						 unsigned char ucFlags=0);

	//Ctor and Dtor
	CTCPCrafter();
	virtual ~CTCPCrafter();
protected:
	//Create an IP header
	virtual LPIpHeader ConstructIPHeader (unsigned char  ucProtocol,
										  unsigned short usFragmentationFlags,
										  unsigned char  ucTTL,
										  unsigned short usIdentification,
										  unsigned char  ucHeaderLength)const;

	//Last stop before sending the header
	virtual void FinalIPHeader(LPIpHeader lpHead)const;

	//Create the TCP header
	virtual LPTCPHeader ConstructTCPHeader(unsigned short usSourcePort,
										   unsigned short usDestinationPort,
										   unsigned char ucHeaderLength)const;

	//Last call before send
	virtual void FinalTCPHeader(LPTCPHeader lpHead)const;
private:
	//No copy Ctor
	CTCPCrafter(const CTCPCrafter& rCrafter);

	//No assignment operator
	CTCPCrafter& operator=(const CTCPCrafter& rCrafter);

	//Our sequence number
	unsigned int m_uiSequenceNumber;

	//Our acknowledge number
	unsigned int m_uiAcknowledgeNumber;

	//TCP data offset
	unsigned char m_ucDataOffset;

	//TCP flags
	unsigned char m_ucFlags;

	//Window size
	unsigned short m_usWindow;

	//TCP checksum
	unsigned short m_usTCPChecksum;
	BOOL m_bTCPUseDefaultChecksum;

	//Urgent pointer indicator
	unsigned short m_usUrgentPointer;
};

KOMODIA_NAMESPACE_END

#endif // !defined(AFX_TCPCRAFTER_H__E2C01EE5_8ABA_4D74_91D6_E80D1D404E5C__INCLUDED_)
