/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#include "stdafx.h"
#include "TCPSocket.h"

#include "ErrorHandlerMacros.h"
#include "Array_ptr.h"

#include <memory>

#ifdef _MEMORY_DEBUG 
	#define new	   DEBUG_NEW  
	#define malloc DEBUG_MALLOC  
    static char THIS_FILE[] = __FILE__;  
#endif

KOMODIA_NAMESPACE_START

//----------------------- CTCPOptions start -----------------------

#define CTCPOptions_LOGNAME "CTCPOptions"

void CTCPOptions::Reset()
{
	try
	{
		CIPOptions::Reset();
	}
	ERROR_HANDLER("Reset")
}

void CTCPOptions::SetAutoPad(BOOL bAutoPAD)
{
	try
	{
		CIPOptions::SetAutoPad(bAutoPAD);
	}
	ERROR_HANDLER("SetAutoPad")
}

void CTCPOptions::AddOption_ENDLIST()
{
	try
	{
		//Add option end list
		OptionType aOptionType;

		//Get the option
		aOptionType=TCPOptions_END;

		//Add it to buffer
		AddToBuffer((char*)&aOptionType,sizeof(aOptionType));
	}
	ERROR_HANDLER("AddOption_ENDLIST")
}

int CTCPOptions::GetBufferLength()const
{
	try
	{
		return CIPOptions::GetBufferLength();
	}
	ERROR_HANDLER_RETURN("GetBufferLength",0)
}

const char* CTCPOptions::GetBuffer()const
{
	try
	{
		return CIPOptions::GetBuffer();
	}
	ERROR_HANDLER_RETURN("GetBuffer",NULL)
}

void CTCPOptions::AddOption_Nothing()
{
	try
	{
		//Add option do nothing
		OptionType aOptionType;

		//Get the option
		aOptionType=TCPOptions_NO_OPERATION;

		//Add it to buffer
		AddToBuffer((char*)&aOptionType,sizeof(aOptionType));
	}
	ERROR_HANDLER("AddOption_Nothing")
}

CTCPOptions::CTCPOptions() : CIPOptions()
{
	try
	{
		SetName(CTCPOptions_LOGNAME);
	}
	ERROR_HANDLER("CTCPOptions")
}

CTCPOptions::CTCPOptions(const CTCPOptions& rOptions) : CIPOptions(rOptions)
{
	try
	{
		SetName(CTCPOptions_LOGNAME);
	}
	ERROR_HANDLER("CTCPOptions")
}

CTCPOptions::~CTCPOptions()
{
}

void CTCPOptions::AddOption_SegmentSize(unsigned short usMax)
{
	try
	{
		//Add option Max segment
		OptionType aOptionType;

		//Get the option
		aOptionType=TCPOptions_MAX_Segment;

		//Add it to buffer
		AddToBuffer((char*)&aOptionType,
					sizeof(aOptionType));

		//Add length
		aOptionType=TCPOptions_MAX_Segment_Length;
		AddToBuffer((char*)&aOptionType,
					sizeof(aOptionType));

		//Add segment size
		unsigned short usOT;
		usOT=htons(usMax);

		//Add it
		AddToBuffer((char*)&usOT,
					sizeof(usOT));
	}
	ERROR_HANDLER("AddOption_SegmentSize")
}

//----------------------- CTCPOptions end -----------------------

#define CTCPSocket_LOGNAME "CTCPSocket"

unsigned int CTCPSocket::m_uiSequence=(unsigned int)GetCurrentProcessId();

CTCPSocket::CTCPSocket(BOOL bRawSocket) : CSpoofSocket(bRawSocket)
{
	try
	{
		//Set our name
		SetName(CTCPSocket_LOGNAME);

		//Initialize the TCP
		InitializeTCP();
	}
	ERROR_HANDLER("CTCPSocket")
}

CTCPSocket::CTCPSocket(SOCKET aSocket) : CSpoofSocket(aSocket)
{
	try
	{
		//Set our name
		SetName(CTCPSocket_LOGNAME);

		//Initialize all the data
		InitializeTCP();
	}
	ERROR_HANDLER("CTCPSocket")
}

CTCPSocket::~CTCPSocket() 
{
	try
	{
		//Delete the options
		SetTCPOptions(FALSE);
	}
	ERROR_HANDLER("~CTCPSocket")
}

BOOL CTCPSocket::Create()
{
	try
	{
		//Initialize our data
		InitializeTCP();

		//Create it
		SetProtocol(IPPROTO_TCP);

		//Are we raw?
		if (IsRaw())
			return CSpoofSocket::Create(IPPROTO_IP); //Can't create working raw TCP socket
		else
			return CSpoofSocket::Create(IPPROTO_TCP);
	}
	ERROR_HANDLER_RETURN("Create",FALSE)
}

BOOL CTCPSocket::Connect(IP aDestinationAddress,
						 unsigned short usDestinationPort)
{
	try
	{
		return Connect(0,
					   aDestinationAddress,
					   usDestinationPort);
	}
	ERROR_HANDLER_RETURN("Connect",FALSE)
}

BOOL CTCPSocket::Connect(const std::string& rDestinationAddress,
						 unsigned short usDestinationPort)
{
	try
	{
		return Connect(0,
					   rDestinationAddress,
					   usDestinationPort);
	}
	ERROR_HANDLER_RETURN("Connect",FALSE)
}

BOOL CTCPSocket::Connect(unsigned short usSourcePort,
						 IP aDestinationAddress,
						 unsigned short usDestinationPort)
{
	try
	{
		//Quit if not ok
		if (!CheckSocketValid())
			return FALSE;

		///Special handling for raw sockets
		if (IsRaw())
			return SendRaw(usSourcePort,
						   aDestinationAddress,
						   usDestinationPort,
						   NULL,
						   0,
						   TCPFlag_SYN);
		else
		{
			//Set async notification
			int iResult;
			
			//Create the address
			sockaddr_in aSrc;
		
			//Set to 0
			memset(&aSrc,
				   0,
				   sizeof(aSrc));
			aSrc.sin_family=AF_INET;
			aSrc.sin_addr.s_addr=aDestinationAddress;
			aSrc.sin_port=htons(usDestinationPort);

			//Connect
			iResult=connect(GetHandle(),
						    (sockaddr*)&aSrc,
							sizeof(aSrc));

			//Did we have a blocked error
			BOOL bBlocked;
			bBlocked=FALSE;

			//Check the result
			if (iResult==GetErrorCode())
			{
				//Check is it blocking error so we can ignore
				if (WSAGetLastError()!=WSAEWOULDBLOCK)
					SetLastError("Connect");
				else
				{
					//Indicate it's blocked error
					bBlocked=TRUE;
					iResult=!GetErrorCode();
				}
			}
			else
				//Report the error
				SetLastError("Connect");
			
			if (iResult!=GetErrorCode())
			{
				//Check if we are a sync socket
				if (!IsAsyncClass())
					SetConnectionStatus(TRUE);

				//Save where we are connected
				SetConnectedTo(aSrc);
			}

			//Done
			if (!bBlocked)
				return iResult!=GetErrorCode();
			else
				return FALSE;
		}
	}
	ERROR_HANDLER_RETURN("Connect",FALSE)
}

BOOL CTCPSocket::Connect(unsigned short usSourcePort, 
						 const std::string& rDestinationAddress, 
						 unsigned short usDestinationPort)
{
	try
	{
		//Quit if not ok
		if (!CheckSocketValid())
			return FALSE;

		//Delegate the call
		return Connect(usSourcePort,
					   StringToLong(rDestinationAddress),
					   usDestinationPort);
	}
	ERROR_HANDLER_RETURN("Connect",FALSE)
}

LPTCPHeader CTCPSocket::ConstructTCPHeader(unsigned short usSourcePort,
										   unsigned short usDestinationPort,
										   unsigned char ucHeaderLength)const
{
	try
	{
		//Construct the header
		LPTCPHeader lpHead=new _TCPHeader;
		
		//Set source and destination port
		lpHead->usSourcePort=htons(usSourcePort);
		lpHead->usDestinationPort=htons(usDestinationPort);

		//No checksums yet
		lpHead->usChecksum=0;

		//Set windows to 3.0k
		lpHead->usWindows=htons(512);

		//Set the packet number
		lpHead->ulAcknowledgeNumber=0;

		//And the sequence
		lpHead->ulSequenceNumber=htonl(m_uiSequence++);

		//Data offset
		lpHead->ucDataOffset=(ucHeaderLength >> 2) << 4;

		//Flags
		lpHead->ucFlags=0;

		//Urgent pointer
		lpHead->usUrgentPointer=0;

		//Return it to the user
		return lpHead;
	}
	ERROR_HANDLER_RETURN("ConstructTCPHeader",NULL)
}

void CTCPSocket::SetHeaderFlag(LPTCPHeader lpHead,unsigned char ucFlags)
{
	//Logical or
	lpHead->ucFlags=ucFlags;	
}

void CTCPSocket::SetTCPOptions(BOOL bOptions)
{
	try
	{
		//Do we want options, normaly not
		m_bOptions=bOptions;

		//Do we need to delete old options
		if (m_pTCPOptions)
		{
			delete m_pTCPOptions;
			m_pTCPOptions=NULL;
		}

		if (bOptions)
			m_pTCPOptions=new CTCPOptions;
	}
	ERROR_HANDLER("SetTCPOptions")
}

CTCPOptions* CTCPSocket::GetTCPOptions() const
{
	return m_pTCPOptions;
}

BOOL CTCPSocket::Listen(unsigned long ulBackLog)
{
	try
	{
		//Quit if not ok
		if (!CheckSocketValid())
			return FALSE;

		//Try to listen
		int iResult;
		iResult=listen(GetHandle(),
					   ulBackLog);

		//Is all OK
		if (iResult)
		{
			//Report it
			SetLastError("Listen");

			//Exit
			return FALSE;
		}
		else
			return TRUE;
	}
	ERROR_HANDLER_RETURN("Listen",FALSE)
}

CTCPSocket* CTCPSocket::Accept()
{
	try
	{
		//Quit if not ok
		if (!CheckSocketValid())
			return FALSE;

		//First accept the socket
		SOCKET aSocket;
		
		//Where we are connected to
		sockaddr_in aConnected;
		
		//Size of the structure
		int iSize;
		iSize=sizeof(aConnected);

		//Accept it
		aSocket=accept(GetHandle(),
					   (sockaddr*)&aConnected,
					   &iSize);

		//Is all OK
		if (aSocket!=INVALID_SOCKET)
		{
			//Create the new tcp socket
			CTCPSocket* pSocket;
			pSocket=new CTCPSocket(aSocket);

			//Set the address
			pSocket->SetConnectedTo(aConnected);
			pSocket->SetConnectionStatus(TRUE);

			//Done
			return pSocket;
		}
		else
		{
			//Error
			SetLastError("Accept");

			//Exit
			return NULL;
		}
	}
	ERROR_HANDLER_RETURN("Accept",NULL)
}

void CTCPSocket::InitializeTCP()
{
	try
	{
		//We are not connected
		m_bConnected=FALSE;

		//No options
		m_pTCPOptions=NULL;
		SetTCPOptions(FALSE);
	}
	ERROR_HANDLER("InitializeTCP")
}

BOOL CTCPSocket::Accept(CTCPSocket* pNewSocket)
{
	try
	{
		//Quit if not ok
		if (!CheckSocketValid())
			return FALSE;

		//First accept the socket
		SOCKET aNewSocket;
		
		//Where we are connected to
		sockaddr_in aAddress;
		
		//Size of the structure
		int iSize;
		iSize=sizeof(aAddress);

		//Accept it
		aNewSocket=accept(GetHandle(),
						  (sockaddr*)&aAddress,
						  &iSize);

		//Is it OK
		if (aNewSocket!=INVALID_SOCKET)
		{
			//Call before accept routing
			pNewSocket->BeforeAccept();

			//Set the socket data
			pNewSocket->SetConnectedTo(aAddress);
			pNewSocket->AssignSocket(aNewSocket);
			pNewSocket->SetConnectionStatus(TRUE);
			pNewSocket->Accepted();

			//Exit
			return TRUE;
		}
		else
		{
			//Error
			SetLastError("Accept");

			//Exit
			return FALSE;
		}
	}
	ERROR_HANDLER_RETURN("Accept",FALSE)
}

int CTCPSocket::Send(const char *pBuffer,
					 unsigned long ulBufferLength)
{
	try
	{
		//Quit if not ok
		if (!CheckSocketValid())
			return FALSE;

		//Send the data
		int iResult;

		//And send it
		iResult=send(GetHandle(),
					 pBuffer,
					 ulBufferLength,
					 NULL);

		//Did we succeed ?
		if (iResult==GetErrorCode())
			//Set the error code
			SetLastError("Send");

		//Done
		return iResult;
	}
	ERROR_HANDLER_RETURN("Send",FALSE)
}

void CTCPSocket::Accepted()
{
}

void CTCPSocket::FinalTCPHeader(LPTCPHeader lpHead)const
{
}

int  CTCPSocket::SendRaw(unsigned short usSourcePort, 
						 const std::string& rDestinationAddress, 
						 unsigned short usDestinationPort,
						 const char* pBuffer,
						 unsigned long ulBufferLength,
						 unsigned char ucFlags)
{
	try
	{
		//Quit if not ok
		if (!CheckSocketValid())
			return FALSE;

		return SendRaw(usSourcePort,
					   StringToLong(rDestinationAddress),
					   usDestinationPort,
					   pBuffer,
					   ulBufferLength,
					   ucFlags);
	}
	ERROR_HANDLER_RETURN("SendRaw",GetErrorCode())
}

int  CTCPSocket::SendRaw(unsigned short usSourcePort, 
						 IP aDestinationAddress, 
						 unsigned short usDestinationPort,
						 const char* pBuffer,
						 unsigned long ulBufferLength,
						 unsigned char ucFlags)
{
	try
	{
		//Quit if not ok
		if (!CheckSocketValid())
			return FALSE;

		if (IsRaw())
		{
			//Let's try our first attack
			LPTCPHeader lpHead;

			//Header length
			int iHeaderLength;
			iHeaderLength=TCPHeaderLength;

			//If we have TCP options
			if (m_bOptions)
				iHeaderLength+=m_pTCPOptions->GetBufferLength();

			//Create the header
			lpHead=ConstructTCPHeader(usSourcePort,
									  usDestinationPort,
									  iHeaderLength);

			//Protect the header
			std::auto_ptr<TCPHeader> pProtection(lpHead);

			//Set the flags
			if (ucFlags)
				//Set the flags
				SetHeaderFlag(lpHead,ucFlags);
			
			//Result 
			int iResult;

			//Construct diffrently if we have options
			if (m_bOptions)
			{
				//Allocate the data
				char* pOptionBuffer;
				pOptionBuffer=new char[iHeaderLength+ulBufferLength];

				//Protect the buffer
				CArray_ptr<char> pBufferProtection(pOptionBuffer);

				//Copy header
				memcpy(pOptionBuffer,
					   lpHead,
					   TCPHeaderLength);
			
				//Copy options
				memcpy(pOptionBuffer+TCPHeaderLength,
					   m_pTCPOptions->GetBuffer(),
					   m_pTCPOptions->GetBufferLength());

				//Do we have the buffer
				if (ulBufferLength)
					//Copy the data
					memcpy(pOptionBuffer+
						     TCPHeaderLength+
						     m_pTCPOptions->GetBufferLength(),
						   pBuffer,
						   ulBufferLength);

				//Calculate the total length
				int iTotalLength;
				iTotalLength=iHeaderLength+ulBufferLength;

				//Change the header
				lpHead->ucDataOffset=(iTotalLength >> 2) << 4;

				//Checksum it
				lpHead->usChecksum=CalculatePseudoChecksum(pOptionBuffer,
														   iTotalLength,
														   aDestinationAddress,
														   iTotalLength);
				
				//Last change to the header
				FinalTCPHeader(lpHead);

				//Recopy header
				memcpy(pOptionBuffer,
					   lpHead,
					   TCPHeaderLength);

				//Send the data
				iResult=CSpoofSocket::Send(aDestinationAddress,
										   pOptionBuffer,
										   iHeaderLength,
										   usDestinationPort);
			}
			else
			{
				//Our total length
				unsigned long ulTotalLength;
				ulTotalLength=iHeaderLength+ulBufferLength;

				//Allocate the buffer
				char* pNewBuffer;
				pNewBuffer=new char[ulTotalLength];

				//Protect the buffer
				CArray_ptr<char> pBufferProtection(pNewBuffer);

				//Copy the header
				memcpy(pNewBuffer,
					   lpHead,
					   ulTotalLength);

				//Copy the data
				if (ulBufferLength)
					//Copy the data
					memcpy(pNewBuffer+iHeaderLength,
						   pBuffer,
						   ulBufferLength);

				//Calculate the checksum
				lpHead->usChecksum=CalculatePseudoChecksum(pNewBuffer,
														   ulTotalLength,
														   aDestinationAddress,
														   ulTotalLength);

				//Last change to the header
				FinalTCPHeader(lpHead);

				//Copy the header
				memcpy(pNewBuffer,
					   lpHead,
					   iHeaderLength);

				//Send the data
				iResult=CSpoofSocket::Send(aDestinationAddress,
										   pNewBuffer,
										   ulTotalLength,
										   usDestinationPort);
			}

			//Set the last error
			SetLastError("Connect");

			//Exit
			return iResult;
		}
		else
		{
			//Report it
			ReportError("SendRaw","Packet not in raw mode!");

			//Exit
			return GetErrorCode();
		}
	}
	ERROR_HANDLER_RETURN("SendRaw",GetErrorCode())
}

void CTCPSocket::SetConnectionStatus(BOOL bConnected)
{
	m_bConnected=bConnected;
}

BOOL CTCPSocket::IsConnected()const
{
	return m_bConnected;
}

BOOL CTCPSocket::IsAsyncClass()const
{
	return FALSE;
}

BOOL CTCPSocket::SetNagle(BOOL bNagle)
{
	try
	{
		//Quit if not ok
		if (!CheckSocketValid())
			return FALSE;

		//Invert the flag (convert to true bool)
		if (bNagle)
			bNagle=FALSE;
		else
			bNagle=TRUE;

		//Try to set the option
		if (setsockopt(GetHandle(),
					   IPPROTO_TCP,
					   TCP_NODELAY,
					   (const char*)&bNagle,
					   sizeof(bNagle))==GetErrorCode())
		{
			//Report it
			ReportError("SetNagle","Failed to set nagle");

			//Set it
			SetLastError("SetNagle");
		
			//Exit
			return FALSE;
		}

		//Done
		return TRUE;
	}
	ERROR_HANDLER_RETURN("SetNagle",FALSE)
}

BOOL CTCPSocket::Close()
{
	try
	{
		//Quit if not ok
		if (!CheckSocketValid())
			return FALSE;

		//Check if the father can close
		if (CSpoofSocket::Close())
		{
			//Set the connection status
			SetConnectionStatus(FALSE);

			//And exit
			return TRUE;
		}
		else
			return FALSE;
	}
	ERROR_HANDLER_RETURN("Close",FALSE)
}

void CTCPSocket::BeforeAccept()
{
}

KOMODIA_NAMESPACE_END

