/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#if !defined(AFX_TCPSOCKETASYNC_H__1D78325D_C154_4B21_97E5_320394FD47EF__INCLUDED_)
#define AFX_TCPSOCKETASYNC_H__1D78325D_C154_4B21_97E5_320394FD47EF__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "TCPSocket.h"
#include "AsyncSocket.h"

KOMODIA_NAMESPACE_START

#define CONNECTION_TIMEOUT 1

class CTCPSocketAsync :	public CTCPSocket, 
						public CAsyncSocket
{
public:
	//Set a connection timeout
	BOOL SetConnectionTimeout(unsigned long ulMS);

	//Send data over the sockets
	//Send may fail because the system isn't ready to 
	//send the data, check out the value of the send
	//SOCKET_ERROR - Failure
	//0 - Send couldn't finish because it would block
	//    if there was a blocking buffer, it was queued
	//	  (check out AllowBlockedBuffer in CAsyncSocket)
	//Value - Bytes sent 
	virtual int Send(const char* pBuffer,
					 unsigned long ulBufferLength);

	//Close the socket
	virtual BOOL Close();

	//Recieve data from remote socket, can be used with all
	//sub sockets (protocols)
	//Return value:
	//Positive - The number of bytes received.
	//Zero - No data to receive.
	//Negative - Error
	virtual int Receive(char* pBuffer,
						unsigned long ulBufferLength);

	//Connect with default source port as zero
	//bDisableAsync - Connect blockingly to destination
	//				  (user event will not be called)
	//bForceErrorEvent - Will call OnSocketConnect event if an
	//					 immediate error occured (only if bDisableAsync==FALSE)
	virtual BOOL Connect(IP aDestinationAddress,
						 unsigned short usDestinationPort,
						 BOOL bDisableAsync=FALSE,
						 BOOL bForceErrorEvent=FALSE);
	virtual BOOL Connect(const std::string& rDestinationAddress,
						 unsigned short usDestinationPort,
						 BOOL bDisableAsync=FALSE,
						 BOOL bForceErrorEvent=FALSE);

	//Our async connection
	virtual BOOL Connect(unsigned short usSourcePort,
						 IP aDestinationAddress,
						 unsigned short usDestinationPort,
						 BOOL bDisableAsync=FALSE,
						 BOOL bForceErrorEvent=FALSE);
	virtual BOOL Connect(unsigned short usSourcePort,
						 const std::string& rDestinationAddress,
						 unsigned short usDestinationPort,
						 BOOL bDisableAsync=FALSE,
						 BOOL bForceErrorEvent=FALSE);

	//Listen to incoming connections
	virtual BOOL Listen(unsigned long ulBackLog=5);
	
	//Create as a raw socket
	virtual BOOL Create();

	//Ctor and Dtor
	CTCPSocketAsync(BOOL bRawSocket);
	virtual ~CTCPSocketAsync();
protected:
	//Send but don't add to buffer
	virtual int SendNoAdd(const char* pBuffer,
						  unsigned long ulBufferLength);

	//Indicate if we are a blocked socket or an async one
	virtual BOOL IsAsyncClass()const;

	//Go to async mode
	virtual BOOL SetAsync();

	//Get the socket handle
	virtual SOCKET GetAsyncHandle()const;

	//Send data from the send buffer
	virtual BOOL SendBlockedBuffer(const CBlockedBuffer::CBlockedData& rData);
private:
	//No copy Ctor
	CTCPSocketAsync(const CTCPSocketAsync& rSocket);

	//Socket is connected
	virtual BOOL SocketConnected(int iErrorCode);

	//Socket it closed
	virtual BOOL SocketClosed(int iErrorCode);

	//When we have data to write
	virtual BOOL SocketWrite(int iErrorCode);

	//When the socket is accepted, what to do
	virtual void Accepted();

	//When we have a system timeout
	virtual void SystemTimeout();

	//Called before we accept the socket
	virtual void BeforeAccept();

	//Is this a disabled connect
	BOOL m_bDisabledConnect;

	//The timeout
	unsigned long m_ulTimeout;
};

KOMODIA_NAMESPACE_END

#endif // !defined(AFX_TCPSOCKETASYNC_H__1D78325D_C154_4B21_97E5_320394FD47EF__INCLUDED_)
